%% Reproduce results of STDLR-SPIRiT in the follwing paper:
%% Xinlin Zhang, Di Guo, Yiman Huang, Ying Chen, Liansheng Wang, Feng Huang, Qin Xu, Xiaobo Qu, Image Reconstruction with Low-rankness and Self-consistency of k-space Data in Parallel MRI, Medical Image Analysis, DOI: 10.1016/j.media.2020.101687, 2020.
%% Created by Xinlin Zhang
%% Corresponding Author: Xiaobo Qu
%% Homepage: http://csrc.xmu.edu.cn
%% Email: quxiaobo <|at|> xmu.edu.cn
%% Affiliations: Computational Sensing Group (CSG), Departments of Electronic Science, Xiamen University, Xiamen 361005, China
%% All rights are reserved by CSG.
%% 2020-03
%%
clc
clear all
close all
warning off
%%
currentFolder = pwd;
addpath(genpath(currentFolder));
%% load Data
load data_slice10_4coil_cut
load mask_GC_34
load brain_mask_1
%%
im_ori_sos = sos(im_ori);
im_ori = im_ori ./ max(im_ori_sos(:));
[row, col, coil] = size(im_ori);

k_ori = fft2c(im_ori);

%% sampling
Sampling_rate    = numel(find(mask==1)) ./ numel(mask)
k_sampled        = mask .* k_ori;

[CalibSize, ~] = getCalibSize(mask);
kCalib = crop(k_sampled,[CalibSize, coil]);

%% recon by STDLR-SPIRiT
% calculate weights
W_hor = Cal_weights_hor(row,col);
W_ver = Cal_weights_ver(row,col);

% calculate kernels
kSize_STDLR_SPIRiT = [5,5];
CalibTyk_STDLR_SPIRiT = 0.01;

kernel_STDLR_SPIRiT = zeros([kSize_STDLR_SPIRiT, coil, coil]);
[AtA,] = dat2AtA(kCalib, kSize_STDLR_SPIRiT);
for n = 1 : coil
    kernel_STDLR_SPIRiT(:,:,:,n) = calibrate(AtA, kSize_STDLR_SPIRiT, coil, n, CalibTyk_STDLR_SPIRiT);
end
GOP_STDLR_SPIRiT = SPIRiT(kernel_STDLR_SPIRiT, 'fft',[row,col]);

% parameters setting
params_STDLR_SPIRiT.row      = row;
params_STDLR_SPIRiT.col      = col;
params_STDLR_SPIRiT.coil     = coil;
params_STDLR_SPIRiT.lambda_2 = 1e6;
params_STDLR_SPIRiT.lambda_1 = 1e4;
params_STDLR_SPIRiT.Tol      = 1*1e-6;
params_STDLR_SPIRiT.beta_hor = 1;
params_STDLR_SPIRiT.beta_ver = 1;
params_STDLR_SPIRiT.Ima_ori  = k_ori;
params_STDLR_SPIRiT.size_PQ  = 400;
params_STDLR_SPIRiT.k1       = 23;
params_STDLR_SPIRiT.k2       = 23;
params_STDLR_SPIRiT.MaxIter  = 80;

tic
out_STDLR_SPIRiT  = Solver_STDLR_SPIRiT(k_sampled, mask, GOP_STDLR_SPIRiT, params_STDLR_SPIRiT, W_hor, W_ver);
runtime_STDLR_SPIRiT = toc

k_rec_STDLR_SPIRiT = out_STDLR_SPIRiT.X_rec;
im_rec_STDLR_SPIRiT = ifft2c(k_rec_STDLR_SPIRiT);

%% RLNE & MSSIM
RLNE_STDLR_SPIRiT = rlne(im_ori, im_rec_STDLR_SPIRiT)

im_ori_ssim = sos(im_ori);
im_ori_ssim = im_ori_ssim .* ssim_mask;
im_rec_STDLR_SPIRiT_ssim = sos(im_rec_STDLR_SPIRiT);
im_rec_STDLR_SPIRiT_ssim = im_rec_STDLR_SPIRiT_ssim .* ssim_mask;

MSSIM_STDLR_SPIRiT = ssim((im_ori_ssim),(im_rec_STDLR_SPIRiT_ssim))